@@ -1,6 +1,8 @@
 package Test::MockDBI;
+
 # Test DBI interfaces using Test::MockObject.
 
+# $Id: MockDBI.pm 244 2008-12-04 12:57:07Z aff $
 
 # ------ use/require pragmas
 use 5.008;                              # minimum Perl is V5.8.0
@@ -23,8 +25,7 @@ our @EXPORT_OK                          # symbols to export upon request
  = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();                     # symbols to always export
 our @ISA = qw(Exporter);                # we ISA Exporter :)
-our $VERSION = '0.61';                  # our version number
-
+our $VERSION = '0.62';                  # our version number
 
 # ------ file-global variables
 my %array_retval  = ();                 # return array values for matching SQL
@@ -41,10 +42,11 @@ my $object        = "";                 # our fake DBI object
 my %rows_retval   = ();                 # return DBI::rows() values for matching SQL
 my %scalar_retval = ();                 # return scalar values for matching SQL
 my $type          = 0;                  # DBI testing type from command line
-
+my %errstr        = ();                 # The scalar to return for errors
+my $debug         = undef;              # Toggle to enable debugging
 
 # ------ convert argument to defined value, use "" if undef argument
-sub Define {
+sub _define {
     my $arg = shift;                    # argument to convert
 
     if (defined($arg)) {
@@ -55,9 +57,9 @@ sub Define {
 
 
 # ------ return TRUE if SQL matches pattern, handle undef values
-sub sql_match {
-    my $sql     = Define(shift);        # SQL
-    my $pattern = Define(shift);        # SQL regex string to match
+sub _sql_match {
+    my $sql     = _define(shift);        # SQL
+    my $pattern = _define(shift);        # SQL regex string to match
 
     if (!$sql && !$pattern) {
         return 1;
@@ -73,7 +75,7 @@ sub sql_match {
 
 
 # ------ check if this DBI method should fail
-sub fail {
+sub _fail {
     my $method  = shift;                # method name
     my $spec    = "";                   # method failure specification
 
@@ -96,12 +98,12 @@ sub fail {
     }
 
     # ------ return TRUE if SQL matches
-    return sql_match($cur_sql, $spec->{$type}->{"SQL"});
+    return _sql_match($cur_sql, $spec->{$type}->{"SQL"});
 }
 
 
 # ------ bind an array to DBI columns bound by bind_columns()
-sub bind_array {
+sub _bind_array {
     my $i;                              # generic loop index
 
     return if (scalar(@bind_columns == 0));
@@ -113,35 +115,41 @@ sub bind_array {
 
 
 # ------ force an array return value
-sub force_retval_array {
+sub _force_retval_array {
     local $_;                           # localized topic
     my @array = ();                     # generic array
 
     foreach (@{ $array_retval{MOCKDBI_WILDCARD()} }, @{ $array_retval{$type} }) {
-        if (sql_match($cur_sql, $_->{"SQL"})) {
+        if (_sql_match($cur_sql, $_->{"SQL"})) {
             if (ref($_->{"retval"}) eq "ARRAY"
              && ref($_->{"retval"}->[0]) eq "CODE") {
                 @array = &{ $_->{"retval"}->[0] }();
                 if (scalar(@array) > 0) {
-                    bind_array(@array);
+                    _bind_array(@array);
                 }
                 return @array;
             }
             @array = @{ $_->{"retval"} };
-            bind_array(@array);
+            _bind_array(@array);
+
+            # Return array ref if first element of array is HASH ref
+            if (scalar(@array) && ref($array[0]) eq 'HASH') {
+              (defined($array[0])) ? return \@array : return;
+            }
+
             return @array;
         }
     }
     if (scalar(@_) < 1) {
         return ();
     }
-    bind_array(@_);
+    _bind_array(@_);
     return @_;
 }
 
 
 # ------ bind an arrayref to DBI columns bound by bind_columns()
-sub bind_arrayref {
+sub _bind_arrayref {
     my $i;                              # generic loop index
 
     return if (scalar(@bind_columns == 0));
@@ -158,37 +166,37 @@ sub bind_arrayref {
 
 
 # ------ force a scalar return value
-sub force_retval_scalar {
+sub _force_retval_scalar {
     local $_;                           # localized topic
     my $arrayref = "";                  # (probably) generic arrayref
 
     foreach (@{ $scalar_retval{MOCKDBI_WILDCARD()} }, @{ $scalar_retval{$type} }) {
-        if (sql_match($cur_sql, $_->{"SQL"})) {
+        if (_sql_match($cur_sql, $_->{"SQL"})) {
             if (ref($_->{"retval"}) eq "CODE") {
                 $arrayref = &{ $_->{"retval"} }();
                 if (defined($arrayref) && ref($arrayref) eq "ARRAY") {
-                    bind_arrayref($arrayref);
+                    _bind_arrayref($arrayref);
                 }
                 return $arrayref;
             }
             $arrayref = $_->{"retval"};
-            bind_arrayref($arrayref);
+            _bind_arrayref($arrayref);
             return $arrayref;
         }
     }
     if (defined($_[0])) {
-        bind_arrayref($_[0]);
+        _bind_arrayref($_[0]);
     }
     return $_[0];
 }
 
 
 # ------ force a DBI::rows() return value
-sub force_retval_rows {
+sub _force_retval_rows {
     local $_;                           # localized topic
 
     foreach (@{ $rows_retval{MOCKDBI_WILDCARD()} }, @{ $rows_retval{$type} }) {
-        if (sql_match($cur_sql, $_->{"SQL"})) {
+        if (_sql_match($cur_sql, $_->{"SQL"})) {
             if (ref($_->{"retval"}) eq "CODE") {
                 return &{ $_->{"retval"} }();
             }
@@ -200,32 +208,32 @@ sub force_retval_rows {
 
 
 # ------ fake the specified DBI method call
-sub fake {
+sub _fake {
     my $method = shift;                 # file-global method name
     my $arg    = shift;                 # first method arg
     my $retval;                         # scalar to return
 
-    print "\n$method()";
+    print "\n$method()" if ($debug);
     if (defined($arg)) {
-        print " '$arg'";
+        print " '$arg'" if ($debug);
     }
-    print "\n";
-    if (fail($method)) {
-        return undef;
+    print "\n" if ($debug);
+    if (_fail($method)) {
+      return;
     }
 
     if ($method eq "rows") {
         $retval = shift;
-        return force_retval_rows($retval);
+        return _force_retval_rows($retval);
     } elsif ($method =~ m/^fetch/ || $method =~ m/^select/) {
         if ($method eq "fetch"
          || $method eq "fetchrow"
          || $method eq "fetchrow_array"
          || $method eq "selectrow_array") {
-            return force_retval_array(@_);
+            return _force_retval_array(@_);
         }
         $retval = shift;
-        return force_retval_scalar($retval);
+        return _force_retval_scalar($retval);
     }
 
     $retval = shift;
@@ -261,6 +269,7 @@ sub bad_method {
     my $sql    = shift;                 # SQL pattern for badness
 
     $fail{$method}->{$type}->{"SQL"} = $sql;
+    return 1;
 }
 
 
@@ -282,7 +291,7 @@ sub set_retval_scalar {
     my $sql    = shift;                 # SQL pattern for badness
 
     push @{ $scalar_retval{$type} },
-     { "SQL" => $sql, "retval" => $_[0] },
+     { "SQL" => $sql, "retval" => $_[0] };
 }
 
 
@@ -298,13 +307,45 @@ sub set_rows {
 
 
 # ------ force a parameter to be bad
+# ------ Returns current number of bad params
 sub bad_param {
     my $self      = shift;              # my blessed self
     my $bad_type  = shift;              # type number from --dbitest=TYPE
     my $bad_param = shift;              # "known" bad parameter number
     my $bad_value = shift;              # "known" bad parameter value
 
-        push(@bad_params, [ $bad_type, $bad_param, $bad_value ] );
+    push(@bad_params, [ $bad_type, $bad_param, $bad_value ] );
+}
+
+# ------ allow errstr to be set and unset
+sub set_errstr {
+    my $self      = shift;              # my blessed self
+    my $bad_type  = shift;              # type number from --dbitest=TYPE
+    my $arg       = shift;              # the argument
+    if (defined($arg) && $arg ne '') {
+      $errstr{$bad_type} = $arg;
+    } else {
+      $errstr{$bad_type} = undef;
+    }
+}
+
+# Return true if given param name and value is bad in given mode,
+# otherwise undefined.  Used for testing purposes only.
+sub _is_bad_param {
+  my $self   = shift;    # my blessed self
+  my $type   = shift;    # type number from --dbitest=TYPE
+  my $number = shift;    # "known" bad parameter number
+  my $value  = shift;    # "known" bad parameter value
+
+  foreach my $param (@bad_params) {
+    if ( $param->[0] == $type
+      && $param->[1] == $number
+      && $param->[2] eq $value)
+    {
+      return 1;
+    }
+  }
+  return;
 }
 
 
@@ -329,107 +370,118 @@ if ($type) {
 
     # ------ initialize DBI mock interface
     $mock = Test::MockObject::Extends->new();
-    print "mock DBI interface initialized...\n";
+    print "mock DBI interface initialized...\n" if ($debug);
 
     $mock->fake_module("DBI",
      connect =>  sub {
         my $self = shift;
-        my $dsn  = Define(shift);
-        my $user = Define(shift);
-        my $pass = Define(shift);
+        my $dsn  = _define(shift);
+        my $user = _define(shift);
+        my $pass = _define(shift);
         $object = bless({}, "DBI");
         $cur_sql = "CONNECT TO $dsn AS $user WITH $pass";
         $fail_param = 0;
         @bind_columns = ();
-        return fake("connect", $cur_sql, $object);
+        return _fake("connect", $cur_sql, $object);
      },
      disconnect =>  sub {
         $cur_sql = "DISCONNECT";
         $fail_param = 0;
         @bind_columns = ();
-        return fake("disconnect", $_[1], 1);
+        return _fake("disconnect", $_[1], 1);
      },
      errstr =>  sub {
-        return "DBI error at '$cur_sql'";
+       return _fake("errstr", $_[1], $errstr{$type});
      },
      prepare =>  sub {
-        $cur_sql = Define($_[1]);
+        $cur_sql = _define($_[1]);
         $fail_param = 0;
         @bind_columns = ();
-        return fake("prepare", $_[1], $object);
+        return _fake("prepare", $_[1], $object);
      },
      prepare_cached =>  sub {
-        $cur_sql = Define($_[1]);
+        $cur_sql = _define($_[1]);
         $fail_param = 0;
         @bind_columns = ();
-        return fake("prepare_cached", $_[1], $object);
+        return _fake("prepare_cached", $_[1], $object);
      },
      commit =>  sub {
-        return fake("commit", $_[1], 1);
+        return _fake("commit", $_[1], 1);
      },
      bind_columns =>  sub {
         shift;
         @bind_columns = @_;
-        return fake("bind_columns", $_[0], 1);
+        return _fake("bind_columns", $_[0], 1);
      },
      bind_param => sub {
-        my $self         = shift;         # my blessed self
-        my $param        = Define(shift); # parameter number
-        my $value        = shift;         # parameter value
-        my $attr_or_type = Define(shift); # attributes or type
-        my $bad_param    = "";            # 1 of @bad_params
+        # Return 1 if param bound was good, otherwise -1 (still true,
+        # but indicates badness)
 
-        print "\nbind_param()\n";
-        print "parm $param, value ";
+        my $self         = shift;             # my blessed self
+        my $param        = _define(shift);    # parameter number
+        my $value        = shift;             # parameter value
+        my $attr_or_type = _define(shift);    # attributes or type
+        my $bad_param    = "";                # 1 of @bad_params
+
+        print "\nbind_param()\n" if ($debug);
+        print "parm $param, value " if ($debug);
         print Dumper($value);
         if ($attr_or_type) {
             if (ref($attr_or_type) eq "HASH") {
-                print "  attrs ", Dumper($attr_or_type);
+                print "  attrs ", Dumper($attr_or_type) if ($debug);
             } else {
-                print "type '$attr_or_type'";
+                print "type '$attr_or_type'" if ($debug);
             }
         }
-        print "\n";
-        if (fail("bind_param")) {
-           return undef;
+        print "\n" if ($debug);
+        if (_fail("bind_param")) {
+           return;
         }
+        ## no critic (RequireLexicalLoopIterators)
         foreach $bad_param (@bad_params) {
             if ($bad_param->[0] == $type
              && $bad_param->[1] == $param
              && $bad_param->[2] eq $value) {
-                print "MOCK_DBI: BAD PARAM $param = '$value'\n";
+                print "MOCK_DBI: BAD PARAM $param = '$value'\n" if ($debug);
                 $fail_param = 1;
+                return -1;  # Indicate that param is bad
             }
         }
         return 1;
      },
      do =>  sub {
-        return fake("do", $_[1], 1);
+        return _fake("do", $_[1], 1);
      },
      execute =>  sub {
-        return fake("execute", $_[1], 1);
+        return _fake("execute", $_[1], 1);
      },
      finish =>  sub {
         $fail_param = 0;
-        return fake("finish", $_[1], 1);
+        return _fake("finish", $_[1], 1);
      },
      fetchall_arrayref =>  sub {
-        return fake("fetchall_arrayref", $_[1], undef);
+        return _fake("fetchall_arrayref", $_[1], undef);
      },
      fetchrow_arrayref =>  sub {
-        return fake("fetchrow_arrayref", $_[1], undef);
+        return _fake("fetchrow_arrayref", $_[1], undef);
+     },
+     fetchrow_hashref =>  sub {
+        return _fake("fetchrow_hashref", $_[1], undef);
+     },
+     fetchall_hashref =>  sub {
+        return _fake("fetchall_hashref", $_[1], undef);
      },
      fetchrow_array =>  sub {
-        return fake("fetchrow_array", $_[1]);
+        return _fake("fetchrow_array", $_[1]);
      },
      fetchrow =>  sub {
-        return fake("fetchrow", $_[1]);
+        return _fake("fetchrow", $_[1]);
      },
      fetch =>  sub {
-        return fake("fetch", $_[1]);
+        return _fake("fetch", $_[1]);
      },
      rows =>  sub {
-        return fake("rows", $_[1], 0);
+        return _fake("rows", $_[1], 0);
      },
      );
     $mock->fake_new("DBI");
@@ -439,7 +491,8 @@ if ($type) {
 
 # ------ return our instance, as we are a singleton class
 sub get_instance {
-    return $instance;
+  $debug = shift;
+  return $instance;
 }
 
 
@@ -492,6 +545,44 @@ Test::MockDBI - Mock DBI interface for testing
    $rows || CODEREF);
   $mock_dbi->set_rows(MOCKDBI_WILDCARD, ...
 
+=head1 EXAMPLE
+
+Code: 
+
+  # Enable testing with Test::MockDBI
+  BEGIN { push @ARGV, "--dbitest"; }
+  use Test::MockDBI qw( :all );
+  my $md  = Test::MockDBI::get_instance();
+  my $dbh = DBI->connect("", "", "");
+
+  # Set of return values for given sql query
+  my $aref_of_hrefs = [
+    { name => 'Huey',  instrument => 'cello' },
+    { name => 'Dewey', instrument => 'trombone' },
+    { name => 'Louie', instrument => 'piano' },
+  ];
+  $md->set_retval_scalar(
+    MOCKDBI_WILDCARD,
+    "select name, instrument from nephews",
+    sub { shift @$aref_of_hrefs }
+  );
+
+  # Execute the sql query and fetch results
+  $dbh->prepare("select name, instrument from nephews");
+  while (my $href = $dbh->fetchrow_hashref()) {
+    print $href->{name} .
+          " plays the " .
+          $href->{instrument} . "\n";
+  }
+  __END__
+
+Expected output:
+
+  Huey plays the cello
+  Dewey plays the trombone
+  Louie plays the piano
+
+
 =head1 DESCRIPTION
 
 Test::MockDBI provides a way to test DBI interfaces by
@@ -556,7 +647,7 @@ string (/^\d+$/).
 
 For the DBI method $method_name, when the DBI testing type
 is $dbi_testing_type and the current SQL matches the regex
-pattern in the string $matching_sql, make the function fail
+pattern in the string $matching_sql, make the function _fail
 (usually by returning undef).
 
 =item bad_param()
@@ -577,12 +668,13 @@ will be returned instead.
 
 =item set_retval_scalar()
 
-When the DBI testing type is $dbi_testing_type and
-the current SQL matches the pattern in the string
-$matching_sql, fetchall_arrayref(), fetchrow_arrayref(),
-and fetchrow() return the scalar value $retval.  If retval
-is actually a CODEREF, the scalar returned from calling
-that subroutine will be returned instead.
+When the DBI testing type is $dbi_testing_type and the
+current SQL matches the pattern in the string $matching_sql,
+fetchall_arrayref(), fetchrow_arrayref(), fetchall_hashref(),
+fetchrow_hashref(), and fetchrow() return the scalar value
+$retval . If retval is actually a CODEREF, the scalar
+returned from calling that subroutine will be returned
+instead .
 
 =item set_rows()
 
@@ -592,13 +684,21 @@ $matching_sql, rows() returns the scalar value $rows.
 If retval is actually a CODEREF, the scalar returned from
 calling that subroutine will be returned instead.
 
+=item set_errstr()
+
+Allows I<errstr> to be set and unset at runtime.
+
+=item get_instance()
+
+Returns the Test::MockDBI instance.  This is a singleton.
+Will print debug messages to stdout if given a defined argument.
+
 =back
 
 =head1 NOTES
 
 A good source of Test::MockDBI examples is how the t/*.t
-test programs examine the output (and modify the behavior)
-of the t/*.pl test programs.
+test programs works.
 
 bad_method() forces developers to use a different DBI
 testing type ("--dbitest=TYPE") for each different SQL
@@ -633,9 +733,6 @@ Test::MockDBI (DBD::DBM ships with DBI).
 DBI fetchrow() is supported, although it is so old it
 is no longer documented in the mainline DBI docs.
 
-Test::MockDBI can be viewed as a "printf() and scratch
-head" helper for DBI applications.
-
 =head1 SEE ALSO
 
 DBI, Test::MockObject::Extends, Test::Simple, Test::More,
@@ -648,11 +745,23 @@ applications)
 
 IO::String (for capturing standard output)
 
+=head1 CAVEAT
+
+=head2 fetch*_hashref does not allow modification of returned data
+set.
+
+This means you must copy-by-value if you wish to modify the data
+before returning to the calling client.
+
 =head1 AUTHOR
 
 Mark Leighton Fisher,
 E<lt>mark-fisher@fisherscreek.comE<gt>
 
+Minor modifications (version 0.62 onwards) by
+Andreas Faafeng
+E<lt>aff@cpan.orgE<gt>
+
 =head1 COPYRIGHT
 
 Copyright 2004, Fisher's Creek Consulting, LLC.  Copyright
@@ -662,3 +771,5 @@ Copyright 2004, Fisher's Creek Consulting, LLC.  Copyright
 
 This code is released under the same licenses as Perl
 itself.
+
+=cut
